package uk.ac.cf.spring.demo.user.controller;

import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.print;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import org.springframework.boot.test.mock.mockito.MockBean;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import uk.ac.cf.spring.demo.user.*;

@SpringBootTest
@AutoConfigureMockMvc
//@WebMvcTest(RegistrationController.class)
public class RegistrationControllerMockMVCTest {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private UserRepository userRepository;

    private ObjectMapper objectMapper;


    @BeforeEach
    public void setUp() {
        MockitoAnnotations.openMocks(this);
        objectMapper = new ObjectMapper();  // Initialize ObjectMapper
    }

//    @Test
//    public void testShowRegistrationForm() throws Exception {
//        mockMvc.perform(get("/register"))
//                .andDo(print())
//                .andExpect(status().isOk())
//                .andExpect(view().name("html/register.html"));
//    }

    @Test
    public void testRegisterUser_Success() throws Exception {
        User user = new User(null, "dummy", "dummy@creditsafe.com", "password", "USER");

        when(userRepository.existsByUsername(user.getUsername())).thenReturn(false);
        when(userRepository.existsByEmail(user.getEmail())).thenReturn(false);

        mockMvc.perform(post("/api/users/register")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(objectMapper.writeValueAsString(user)))
                .andExpect(status().isOk())
                .andExpect(content().json("{\"message\": \"Registration successful\"}"));

        verify(userRepository, times(1)).save(user); // Verifying the save method is called once with the user object.
    }

    @Test
    public void testRegisterUser_UsernameExists() throws Exception {
        User user = new User(null, "dummy", "dummy@creditsafe.com", "password", "USER");

        when(userRepository.existsByUsername(user.getUsername())).thenReturn(true);

        mockMvc.perform(post("/api/users/register")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(objectMapper.writeValueAsString(user)))
                .andExpect(status().isBadRequest())
                .andExpect(content().json("{\"message\": \"Username already exists\"}"));

        verify(userRepository, never()).save(user); // Verify that save is not called when username exists.
    }

}

